@@ -5,79 +5,76 @@
 // License, Version 2.0 found in the LICENSE-APACHE file in the root directory
 // of this source tree.
 
+//! An implementation of an append-only zero knowledge set
 use crate::{
     errors::HistoryTreeNodeError,
     history_tree_node::*,
-    storage::{Storable, Storage},
+    proof_structs::{AppendOnlyProof, MembershipProof, NonMembershipProof},
+    storage::{Storable, V2Storage},
 };
 
 use crate::serialization::to_digest;
+
 use crate::storage::types::StorageType;
-use crate::{history_tree_node::HistoryTreeNode, node_state::*, ARITY, *};
+use crate::{errors::*, history_tree_node::HistoryTreeNode, node_state::*, ARITY, *};
 use async_recursion::async_recursion;
-use rand::{CryptoRng, RngCore};
-use std::marker::PhantomData;
+use std::marker::{PhantomData, Send, Sync};
 use winter_crypto::Hasher;
 
 use serde::{Deserialize, Serialize};
 
 use keyed_priority_queue::{Entry, KeyedPriorityQueue};
 
+/// The default azks key
+pub const DEFAULT_AZKS_KEY: u8 = 1u8;
+/// An append-only zero knowledge set, the data structure used to efficiently implement
+/// a auditable key directory.
 #[derive(Debug, Serialize, Deserialize)]
 #[serde(bound = "")]
-pub struct Azks<H, S> {
-    /// Random identifier for the AZKS instance
-    azks_id: [u8; 32],
-    root: usize,
-    latest_epoch: u64,
-    num_nodes: usize, // The size of the tree
-    _s: PhantomData<S>,
-    _h: PhantomData<H>,
+pub struct Azks<H> {
+    /// The location of the root
+    pub root: usize,
+    /// The latest complete epoch
+    pub latest_epoch: u64,
+    /// The number of nodes ie the size of this tree
+    pub num_nodes: usize, // The size of the tree
+    /// Placeholder
+    pub _h: PhantomData<H>,
 }
 
-// parameter is azks_id
-#[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
-pub struct AzksKey(pub(crate) [u8; 32]);
-
-impl<H: Hasher, S: Storage> Storable for Azks<H, S> {
-    type Key = AzksKey;
+impl<H: Hasher> Storable for Azks<H> {
+    type Key = u8;
 
     fn data_type() -> StorageType {
         StorageType::Azks
     }
+
+    fn get_id(&self) -> u8 {
+        1u8
+    }
 }
 
-unsafe impl<H: Hasher, S: Storage> Sync for Azks<H, S> {}
+unsafe impl<H: Hasher> Sync for Azks<H> {}
 
-impl<H: Hasher, S: Storage> Clone for Azks<H, S> {
+impl<H: Hasher> Clone for Azks<H> {
     fn clone(&self) -> Self {
         Self {
-            azks_id: self.azks_id,
             root: self.root,
             latest_epoch: self.latest_epoch,
             num_nodes: self.num_nodes,
-            _s: PhantomData,
             _h: PhantomData,
         }
     }
 }
 
-impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker::Send> Azks<H, S> {
-    pub async fn new<R: CryptoRng + RngCore>(
-        storage: &S,
-        rng: &mut R,
-    ) -> Result<Self, SeemlessError> {
-        let mut azks_id: [u8; 32] = [0u8; 32];
-        rng.fill_bytes(&mut azks_id);
-
-        let root = get_empty_root::<H, S>(storage, &azks_id, Option::Some(0)).await?;
-
+impl<H: Hasher + Send + Sync> Azks<H> {
+    /// Creates a new azks
+    pub async fn new<S: V2Storage + Sync + Send>(storage: &S) -> Result<Self, AkdError> {
+        let root = get_empty_root::<H, S>(storage, Option::Some(0)).await?;
         let azks = Azks {
-            azks_id,
             root: 0,
             latest_epoch: 0,
             num_nodes: 1,
-            _s: PhantomData,
             _h: PhantomData,
         };
 
@@ -86,81 +83,61 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
         Ok(azks)
     }
 
-    pub async fn insert_leaf(
+    /// Inserts a single leaf and is only used for testing, since batching is more efficient.
+    /// We just want to make sure batch insertions work correctly and this function is useful for that.
+    pub async fn insert_leaf<S: V2Storage + Sync + Send>(
         &mut self,
         storage: &S,
         label: NodeLabel,
         value: H::Digest,
-    ) -> Result<(), SeemlessError> {
+    ) -> Result<(), AkdError> {
         // Calls insert_single_leaf on the root node and updates the root and tree_nodes
         self.increment_epoch();
 
-        let new_leaf = get_leaf_node::<H, S>(
-            storage,
-            &self.azks_id,
-            label,
-            0,
-            value.as_ref(),
-            0,
-            self.latest_epoch,
-        )
-        .await?;
+        let new_leaf =
+            get_leaf_node::<H, S>(storage, label, 0, value.as_ref(), 0, self.latest_epoch).await?;
 
-        let mut root_node = storage
-            .retrieve::<HistoryTreeNode<H, S>>(NodeKey(self.azks_id, self.root))
-            .await?;
+        let mut root_node = HistoryTreeNode::get_from_storage(storage, NodeKey(self.root)).await?;
         root_node
-            .insert_single_leaf(
-                storage,
-                new_leaf,
-                &self.azks_id,
-                self.latest_epoch,
-                &mut self.num_nodes,
-            )
+            .insert_single_leaf(storage, new_leaf, self.latest_epoch, &mut self.num_nodes)
             .await?;
 
         Ok(())
     }
 
-    pub async fn batch_insert_leaves(
+    /// Insert a batch of new leaves
+    pub async fn batch_insert_leaves<S: V2Storage + Sync + Send>(
         &mut self,
         storage: &S,
         insertion_set: Vec<(NodeLabel, H::Digest)>,
-    ) -> Result<(), SeemlessError> {
+    ) -> Result<(), AkdError> {
         self.batch_insert_leaves_helper(storage, insertion_set, false)
             .await
     }
 
-    pub async fn batch_insert_leaves_helper(
+    /// An azks is built both by the [crate::directory::Directory] and the auditor.
+    /// However, both constructions have very minor differences, and the append_only_usage
+    /// bool keeps track of this.
+    pub async fn batch_insert_leaves_helper<S: V2Storage + Sync + Send>(
         &mut self,
         storage: &S,
         insertion_set: Vec<(NodeLabel, H::Digest)>,
         append_only_usage: bool,
-    ) -> Result<(), SeemlessError> {
+    ) -> Result<(), AkdError> {
         self.increment_epoch();
 
         let mut hash_q = KeyedPriorityQueue::<usize, i32>::new();
         let mut priorities: i32 = 0;
-        let mut root_node = storage
-            .retrieve::<HistoryTreeNode<H, S>>(NodeKey(self.azks_id, self.root))
-            .await?;
+        let mut root_node = HistoryTreeNode::get_from_storage(storage, NodeKey(self.root)).await?;
         for (label, value) in insertion_set {
             let new_leaf_loc = self.num_nodes;
 
-            let mut new_leaf = get_leaf_node::<H, S>(
-                storage,
-                &self.azks_id,
-                label,
-                0,
-                value.as_ref(),
-                0,
-                self.latest_epoch,
-            )
-            .await?;
+            let mut new_leaf =
+                get_leaf_node::<H, S>(storage, label, 0, value.as_ref(), 0, self.latest_epoch)
+                    .await?;
             if append_only_usage {
                 new_leaf = get_leaf_node_without_hashing::<H, S>(
                     storage,
-                    &self.azks_id,
                     label,
                     0,
                     value,
@@ -171,13 +148,7 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
             }
 
             root_node
-                .insert_single_leaf_without_hash(
-                    storage,
-                    new_leaf,
-                    &self.azks_id,
-                    self.latest_epoch,
-                    &mut self.num_nodes,
-                )
+                .insert_leaf(storage, new_leaf, self.latest_epoch, &mut self.num_nodes)
                 .await?;
 
             hash_q.push(new_leaf_loc, priorities);
@@ -189,9 +160,8 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
                 .pop()
                 .ok_or(AzksError::PopFromEmptyPriorityQueue(self.latest_epoch))?;
 
-            let mut next_node = storage
-                .retrieve::<HistoryTreeNode<H, S>>(NodeKey(self.azks_id, next_node_loc))
-                .await?;
+            let mut next_node: HistoryTreeNode<H> =
+                HistoryTreeNode::get_from_storage(storage, NodeKey(next_node_loc)).await?;
 
             next_node.update_hash(storage, self.latest_epoch).await?;
 
@@ -211,45 +181,42 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
         Ok(())
     }
 
-    pub async fn get_membership_proof(
+    /// Returns the Merkle membership proof for the trie as it stood at epoch
+    // Assumes the verifier has access to the root at epoch
+    pub async fn get_membership_proof<S: V2Storage + Sync + Send>(
         &self,
         storage: &S,
         label: NodeLabel,
         epoch: u64,
-    ) -> Result<MembershipProof<H>, SeemlessError> {
-        // Regular Merkle membership proof for the trie as it stood at epoch
-        // Assumes the verifier as access to the root at epoch
+    ) -> Result<MembershipProof<H>, AkdError> {
         let (pf, _) = self
             .get_membership_proof_and_node(storage, label, epoch)
             .await?;
         Ok(pf)
     }
 
-    pub async fn get_non_membership_proof(
+    /// In a compressed trie, the proof consists of the longest prefix
+    /// of the label that is included in the trie, as well as its children, to show that
+    /// none of the children is equal to the given label.
+    pub async fn get_non_membership_proof<S: V2Storage + Sync + Send>(
         &self,
         storage: &S,
         label: NodeLabel,
         epoch: u64,
-    ) -> Result<NonMembershipProof<H>, SeemlessError> {
-        // In a compressed trie, the proof consists of the longest prefix
-        // of the label that is included in the trie, as well as its children, to show that
-        // none of the children is equal to the given label.
-
+    ) -> Result<NonMembershipProof<H>, AkdError> {
         let (longest_prefix_membership_proof, lcp_node_id) = self
             .get_membership_proof_and_node(storage, label, epoch)
             .await?;
-        let lcp_node = storage
-            .retrieve::<HistoryTreeNode<H, S>>(NodeKey(self.azks_id, lcp_node_id))
-            .await?;
+        let lcp_node: HistoryTreeNode<H> =
+            HistoryTreeNode::get_from_storage(storage, NodeKey(lcp_node_id)).await?;
         let longest_prefix = lcp_node.label;
         let mut longest_prefix_children_labels = [NodeLabel::new(0, 0); ARITY];
         let mut longest_prefix_children_values = [H::hash(&[]); ARITY];
         let state = lcp_node.get_state_at_epoch(storage, epoch).await?;
 
         for (i, child) in state.child_states.iter().enumerate() {
-            let unwrapped_child: HistoryTreeNode<H, S> = storage
-                .retrieve(NodeKey(self.azks_id, child.location))
-                .await?;
+            let unwrapped_child: HistoryTreeNode<H> =
+                HistoryTreeNode::get_from_storage(storage, NodeKey(child.location)).await?;
             longest_prefix_children_labels[i] = unwrapped_child.label;
             longest_prefix_children_values[i] = unwrapped_child
                 .get_value_without_label_at_epoch(storage, epoch)
@@ -264,16 +231,25 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
         })
     }
 
-    pub async fn get_append_only_proof(
+    /// An append-only proof for going from `start_epoch` to `end_epoch` consists of roots of subtrees
+    /// the azks tree that remain unchanged from `start_epoch` to `end_epoch` and the leaves inserted into the
+    /// tree after `start_epoch` and  up until `end_epoch`.
+    /// If there is no errors, this function returns an `Ok` result, containing the
+    ///  append-only proof and otherwise, it returns a [errors::AkdError].
+    ///
+    /// **RESTRICTIONS**: Note that `start_epoch` and `end_epoch` are valid only when the following are true
+    /// * `start_epoch` <= `end_epoch`
+    /// * `start_epoch` and `end_epoch` are both existing epochs of this AZKS
+    pub async fn get_append_only_proof<S: V2Storage + Sync + Send>(
         &self,
         storage: &S,
         start_epoch: u64,
         end_epoch: u64,
-    ) -> Result<AppendOnlyProof<H>, SeemlessError> {
+    ) -> Result<AppendOnlyProof<H>, AkdError> {
         // Suppose the epochs start_epoch and end_epoch exist in the set.
         // This function should return the proof that nothing was removed/changed from the tree
         // between these epochs.
-        let node = storage.retrieve(NodeKey(self.azks_id, self.root)).await?;
+        let node = HistoryTreeNode::get_from_storage(storage, NodeKey(self.root)).await?;
         let (unchanged, leaves) = self
             .get_append_only_proof_helper(storage, node, start_epoch, end_epoch)
             .await?;
@@ -284,13 +260,13 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
     }
 
     #[async_recursion]
-    async fn get_append_only_proof_helper(
+    async fn get_append_only_proof_helper<S: V2Storage + Sync + Send>(
         &self,
         storage: &S,
-        node: HistoryTreeNode<H, S>,
+        node: HistoryTreeNode<H>,
         start_epoch: u64,
         end_epoch: u64,
-    ) -> Result<AppendOnlyHelper<H::Digest>, SeemlessError> {
+    ) -> Result<AppendOnlyHelper<H::Digest>, AkdError> {
         let mut unchanged = Vec::<(NodeLabel, H::Digest)>::new();
         let mut leaves = Vec::<(NodeLabel, H::Digest)>::new();
         if node.get_latest_epoch()? <= start_epoch {
@@ -327,12 +303,11 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
                 if child_node_state.dummy_marker == DummyChildState::Dummy {
                     continue;
                 } else {
-                    let child_node = storage
-                        .retrieve::<HistoryTreeNode<H, S>>(NodeKey(
-                            self.azks_id,
-                            child_node_state.location,
-                        ))
-                        .await?;
+                    let child_node = HistoryTreeNode::get_from_storage(
+                        storage,
+                        NodeKey(child_node_state.location),
+                    )
+                    .await?;
                     let mut rec_output = self
                         .get_append_only_proof_helper(storage, child_node, start_epoch, end_epoch)
                         .await?;
@@ -344,36 +319,31 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
         Ok((unchanged, leaves))
     }
 
-    pub async fn get_consecutive_append_only_proof(
+    // FIXME: these functions below should be moved into higher-level API
+    /// Gets the root hash for this azks
+    pub async fn get_root_hash<S: V2Storage + Sync + Send>(
         &self,
         storage: &S,
-        start_epoch: u64,
-    ) -> Result<AppendOnlyProof<H>, SeemlessError> {
-        // Suppose the epochs start_epoch and start_epoch+1 exist in the set.
-        // This function should return the proof that nothing was removed/changed from the tree
-        // between these epochs.
-        self.get_append_only_proof(storage, start_epoch, start_epoch + 1)
-            .await
-    }
-
-    // FIXME: these functions below should be moved into higher-level API
-
-    pub async fn get_root_hash(&self, storage: &S) -> Result<H::Digest, HistoryTreeNodeError> {
+    ) -> Result<H::Digest, HistoryTreeNodeError> {
         self.get_root_hash_at_epoch(storage, self.get_latest_epoch())
             .await
     }
 
-    pub async fn get_root_hash_at_epoch(
+    /// Gets the root hash of the tree at a epoch.
+    /// Since this is accessing the root node and the root node exists at all epochs that
+    /// the azks does, this would never be called at an epoch before the birth of the root node.
+    pub async fn get_root_hash_at_epoch<S: V2Storage + Sync + Send>(
         &self,
         storage: &S,
         epoch: u64,
     ) -> Result<H::Digest, HistoryTreeNodeError> {
-        let root_node = storage
-            .retrieve::<HistoryTreeNode<H, S>>(NodeKey(self.azks_id, self.root))
-            .await?;
+        let root_node: HistoryTreeNode<H> =
+            HistoryTreeNode::get_from_storage(storage, NodeKey(self.root)).await?;
         root_node.get_value_at_epoch(storage, epoch).await
     }
 
+    /// Gets the latest epoch of this azks. If an update aka epoch transition
+    /// is in progress, this should return the most recent completed epoch.
     pub fn get_latest_epoch(&self) -> u64 {
         self.latest_epoch
     }
@@ -383,19 +353,21 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
         self.latest_epoch = epoch;
     }
 
-    pub async fn get_membership_proof_and_node(
+    /// This function returns the node location for the node whose label is the longest common
+    /// prefix for the queried label. It also returns a membership proof for said label.
+    /// This is meant to be used in both, getting membership proofs and getting non-membership proofs.
+    pub async fn get_membership_proof_and_node<S: V2Storage + Sync + Send>(
         &self,
         storage: &S,
         label: NodeLabel,
         epoch: u64,
-    ) -> Result<(MembershipProof<H>, usize), SeemlessError> {
+    ) -> Result<(MembershipProof<H>, usize), AkdError> {
         let mut parent_labels = Vec::<NodeLabel>::new();
         let mut sibling_labels = Vec::<[NodeLabel; ARITY - 1]>::new();
         let mut sibling_hashes = Vec::<[H::Digest; ARITY - 1]>::new();
         let mut dirs = Vec::<Direction>::new();
-        let mut curr_node = storage
-            .retrieve::<HistoryTreeNode<H, S>>(NodeKey(self.azks_id, self.root))
-            .await?;
+        let mut curr_node: HistoryTreeNode<H> =
+            HistoryTreeNode::get_from_storage(storage, NodeKey(self.root)).await?;
         let mut dir = curr_node.label.get_dir(label);
         let mut equal = label == curr_node.label;
         let mut prev_node = 0;
@@ -407,13 +379,13 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
             let mut labels = [NodeLabel::new(0, 0); ARITY - 1];
             let mut hashes = [H::hash(&[0u8]); ARITY - 1];
             let mut count = 0;
-            let direction = dir.ok_or(SeemlessError::NoDirectionError)?;
+            let direction = dir.ok_or(AkdError::NoDirectionError)?;
             let next_state = curr_state.get_child_state_in_dir(direction);
             if next_state.dummy_marker == DummyChildState::Dummy {
                 break;
             }
             for i in 0..ARITY {
-                if i != dir.ok_or(SeemlessError::NoDirectionError)? {
+                if i != dir.ok_or(AkdError::NoDirectionError)? {
                     labels[count] = curr_state.child_states[i].label;
                     hashes[count] = to_digest::<H>(&curr_state.child_states[i].hash_val).unwrap();
                     count += 1;
@@ -421,20 +393,22 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
             }
             sibling_labels.push(labels);
             sibling_hashes.push(hashes);
-            let new_curr_node = storage
-                .retrieve::<HistoryTreeNode<H, S>>(NodeKey(
-                    self.azks_id,
+            let new_curr_node: HistoryTreeNode<H> = HistoryTreeNode::get_from_storage(
+                storage,
+                NodeKey(
                     curr_node
                         .get_child_location_at_epoch(storage, epoch, dir)
                         .await?,
-                ))
-                .await?;
+                ),
+            )
+            .await?;
             curr_node = new_curr_node;
             dir = curr_node.label.get_dir(label);
             equal = label == curr_node.label;
         }
         if !equal {
-            let new_curr_node = storage.retrieve(NodeKey(self.azks_id, prev_node)).await?;
+            let new_curr_node: HistoryTreeNode<H> =
+                HistoryTreeNode::get_from_storage(storage, NodeKey(prev_node)).await?;
             curr_node = new_curr_node;
 
             parent_labels.pop();
@@ -459,10 +433,6 @@ impl<H: Hasher + std::marker::Send, S: Storage + std::marker::Sync + std::marker
             prev_node,
         ))
     }
-
-    pub fn get_azks_id(&self) -> [u8; 32] {
-        self.azks_id
-    }
 }
 
 type AppendOnlyHelper<D> = (Vec<(NodeLabel, D)>, Vec<(NodeLabel, D)>);
@@ -483,11 +453,11 @@ mod tests {
     type Blake3Digest = <Blake3_256<winter_math::fields::f128::BaseElement> as Hasher>::Digest;
 
     #[actix_rt::test]
-    async fn test_batch_insert_basic() -> Result<(), SeemlessError> {
-        let num_nodes = 10;
+    async fn test_batch_insert_basic() -> Result<(), AkdError> {
         let mut rng = OsRng;
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks1 = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let num_nodes = 10;
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks1 = Azks::<Blake3>::new(&db).await?;
 
         let mut insertion_set: Vec<(NodeLabel, Blake3Digest)> = vec![];
 
@@ -500,13 +470,14 @@ mod tests {
             azks1.insert_leaf(&db, node, val).await?;
         }
 
-        let mut azks2 = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let db2 = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks2 = Azks::<Blake3>::new(&db2).await?;
 
-        azks2.batch_insert_leaves(&db, insertion_set).await?;
+        azks2.batch_insert_leaves(&db2, insertion_set).await?;
 
         assert_eq!(
             azks1.get_root_hash(&db).await?,
-            azks2.get_root_hash(&db).await?,
+            azks2.get_root_hash(&db2).await?,
             "Batch insert doesn't match individual insert"
         );
 
@@ -514,11 +485,11 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_insert_permuted() -> Result<(), SeemlessError> {
+    async fn test_insert_permuted() -> Result<(), AkdError> {
         let num_nodes = 10;
         let mut rng = OsRng;
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks1 = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks1 = Azks::<Blake3>::new(&db).await?;
         let mut insertion_set: Vec<(NodeLabel, Blake3Digest)> = vec![];
 
         for _ in 0..num_nodes {
@@ -533,13 +504,14 @@ mod tests {
         // Try randomly permuting
         insertion_set.shuffle(&mut rng);
 
-        let mut azks2 = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let db2 = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks2 = Azks::<Blake3>::new(&db2).await?;
 
-        azks2.batch_insert_leaves(&db, insertion_set).await?;
+        azks2.batch_insert_leaves(&db2, insertion_set).await?;
 
         assert_eq!(
             azks1.get_root_hash(&db).await?,
-            azks2.get_root_hash(&db).await?,
+            azks2.get_root_hash(&db2).await?,
             "Batch insert doesn't match individual insert"
         );
 
@@ -547,7 +519,7 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_membership_proof_permuted() -> Result<(), SeemlessError> {
+    async fn test_membership_proof_permuted() -> Result<(), AkdError> {
         let num_nodes = 10;
         let mut rng = OsRng;
 
@@ -563,8 +535,8 @@ mod tests {
 
         // Try randomly permuting
         insertion_set.shuffle(&mut rng);
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks = Azks::<Blake3>::new(&db).await?;
         azks.batch_insert_leaves(&db, insertion_set.clone()).await?;
 
         let proof = azks
@@ -577,7 +549,7 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_membership_proof_failing() -> Result<(), SeemlessError> {
+    async fn test_membership_proof_failing() -> Result<(), AkdError> {
         let num_nodes = 10;
         let mut rng = OsRng;
 
@@ -593,8 +565,8 @@ mod tests {
 
         // Try randomly permuting
         insertion_set.shuffle(&mut rng);
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks = Azks::<Blake3>::new(&db).await?;
         azks.batch_insert_leaves(&db, insertion_set.clone()).await?;
 
         let mut proof = azks
@@ -618,16 +590,15 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_membership_proof_intermediate() -> Result<(), SeemlessError> {
-        let mut rng = OsRng;
-        let db = AsyncInMemoryDatabase::new();
+    async fn test_membership_proof_intermediate() -> Result<(), AkdError> {
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
         let mut insertion_set: Vec<(NodeLabel, Blake3Digest)> = vec![];
         insertion_set.push((NodeLabel::new(0b0, 64), Blake3::hash(&[])));
         insertion_set.push((NodeLabel::new(0b1 << 63, 64), Blake3::hash(&[])));
         insertion_set.push((NodeLabel::new(0b11 << 62, 64), Blake3::hash(&[])));
         insertion_set.push((NodeLabel::new(0b01 << 62, 64), Blake3::hash(&[])));
         insertion_set.push((NodeLabel::new(0b111 << 61, 64), Blake3::hash(&[])));
-        let mut azks = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let mut azks = Azks::<Blake3>::new(&db).await?;
         azks.batch_insert_leaves(&db, insertion_set).await?;
         let search_label = NodeLabel::new(0b1111 << 60, 64);
         let proof = azks.get_non_membership_proof(&db, search_label, 1).await?;
@@ -639,7 +610,7 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_nonmembership_proof() -> Result<(), SeemlessError> {
+    async fn test_nonmembership_proof() -> Result<(), AkdError> {
         let num_nodes = 10;
         let mut rng = OsRng;
 
@@ -652,8 +623,8 @@ mod tests {
             let input = Blake3Digest::new(input);
             insertion_set.push((node, input));
         }
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks = Azks::<Blake3>::new(&db).await?;
         let search_label = insertion_set[num_nodes - 1].0;
         azks.batch_insert_leaves(&db, insertion_set.clone()[0..num_nodes - 1].to_vec())
             .await?;
@@ -668,10 +639,9 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_append_only_proof_very_tiny() -> Result<(), SeemlessError> {
-        let mut rng = OsRng;
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+    async fn test_append_only_proof_very_tiny() -> Result<(), AkdError> {
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks = Azks::<Blake3>::new(&db).await?;
 
         let mut insertion_set_1: Vec<(NodeLabel, Blake3Digest)> = vec![];
         insertion_set_1.push((NodeLabel::new(0b0, 64), Blake3::hash(&[])));
@@ -691,10 +661,9 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_append_only_proof_tiny() -> Result<(), SeemlessError> {
-        let mut rng = OsRng;
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+    async fn test_append_only_proof_tiny() -> Result<(), AkdError> {
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks = Azks::<Blake3>::new(&db).await?;
 
         let mut insertion_set_1: Vec<(NodeLabel, Blake3Digest)> = vec![];
         insertion_set_1.push((NodeLabel::new(0b0, 64), Blake3::hash(&[])));
@@ -716,7 +685,7 @@ mod tests {
     }
 
     #[actix_rt::test]
-    async fn test_append_only_proof() -> Result<(), SeemlessError> {
+    async fn test_append_only_proof() -> Result<(), AkdError> {
         let num_nodes = 10;
         let mut rng = OsRng;
 
@@ -730,8 +699,8 @@ mod tests {
             insertion_set_1.push((node, input));
         }
 
-        let db = AsyncInMemoryDatabase::new();
-        let mut azks = Azks::<Blake3, AsyncInMemoryDatabase>::new(&db, &mut rng).await?;
+        let db = storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut azks = Azks::<Blake3>::new(&db).await?;
         azks.batch_insert_leaves(&db, insertion_set_1.clone())
             .await?;
 
