@@ -5,76 +5,77 @@
 // License, Version 2.0 found in the LICENSE-APACHE file in the root directory
 // of this source tree.
 
-use crate::append_only_zks::{Azks, AzksKey};
-use crate::errors::{SeemlessDirectoryError, SeemlessError};
+//! Implementation of a auditable key directory
+
+use crate::append_only_zks::Azks;
 
 use crate::node_state::NodeLabel;
 use crate::proof_structs::*;
-use crate::storage::types::{UserState, UserStateRetrievalFlag, Username, Values};
-use crate::storage::Storage;
 
-use rand::{prelude::ThreadRng, thread_rng};
+use crate::errors::{AkdError, DirectoryError};
+
+use crate::storage::types::{AkdKey, DbRecord, ValueState, ValueStateRetrievalFlag, Values};
+use crate::storage::V2Storage;
+
 use rand::{CryptoRng, RngCore};
 
 use std::collections::HashMap;
-use std::marker::PhantomData;
+use std::marker::{PhantomData, Send, Sync};
 use winter_crypto::Hasher;
 
 impl Values {
+    /// Gets a random value for a AKD
     pub fn random<R: RngCore + CryptoRng>(rng: &mut R) -> Self {
         Self(get_random_str(rng))
     }
 }
 
-impl Username {
+impl AkdKey {
+    /// Creates a random key for a AKD
     pub fn random<R: RngCore + CryptoRng>(rng: &mut R) -> Self {
         Self(get_random_str(rng))
     }
 }
 
+/// The representation of a auditable key directory
 pub struct Directory<S, H> {
-    azks_id: [u8; 32],
     current_epoch: u64,
     storage: S,
-    _s: PhantomData<S>,
     _h: PhantomData<H>,
 }
 
-impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker::Send>
-    Directory<S, H>
-{
-    pub async fn new(storage: &S) -> Result<Self, SeemlessError> {
-        let mut rng: ThreadRng = thread_rng();
-
+impl<S: V2Storage + Sync + Send, H: Hasher + Send + Sync> Directory<S, H> {
+    /// Creates a new (stateless) instance of a auditable key directory.
+    /// Takes as input a pointer to the storage being used for this instance.
+    /// The state is stored in the storage.
+    pub async fn new(storage: &S) -> Result<Self, AkdError> {
         let azks = {
-            if let Some(azks) = Directory::get_azks_from_storage(storage).await {
+            if let Ok(azks) = Directory::get_azks_from_storage(storage).await {
                 azks
             } else {
                 // generate a new one
-                let azks = Azks::<H, S>::new(storage, &mut rng).await?;
+                let azks = Azks::<H>::new(storage).await?;
                 // store it
-                storage.store(AzksKey(azks.get_azks_id()), &azks).await?;
+                storage.set::<H>(DbRecord::Azks(azks.clone())).await?;
                 azks
             }
         };
-        let azks_id = azks.get_azks_id();
         Ok(Directory {
-            azks_id,
             current_epoch: azks.get_latest_epoch(),
-            _s: PhantomData::<S>,
             _h: PhantomData::<H>,
             storage: storage.clone(),
         })
     }
 
-    pub async fn publish(&mut self, updates: Vec<(Username, Values)>) -> Result<(), SeemlessError> {
+    /// Updates the directory to inclulde the updated key-value pairs.
+    pub async fn publish(&mut self, updates: Vec<(AkdKey, Values)>) -> Result<(), AkdError> {
         let mut update_set = Vec::<(NodeLabel, H::Digest)>::new();
-        let mut user_data_update_set = Vec::<(Username, UserState)>::new();
+        let mut user_data_update_set = Vec::<ValueState>::new();
         let next_epoch = self.current_epoch + 1;
         for (uname, val) in updates {
             match self
                 .storage
-                .get_user_state(&uname, UserStateRetrievalFlag::MaxEpoch)
+                .get_user_state::<H>(&uname, ValueStateRetrievalFlag::MaxEpoch)
                 .await
             {
                 Err(_) => {
@@ -85,8 +86,9 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
                     // We'd want to change this later.
                     let value_to_add = H::hash(&Self::value_to_bytes(&val));
                     update_set.push((label, value_to_add));
-                    let latest_state = UserState::new(val, latest_version, label, next_epoch);
-                    user_data_update_set.push((uname, latest_state));
+                    let latest_state =
+                        ValueState::new(uname, val, latest_version, label, next_epoch);
+                    user_data_update_set.push(latest_state);
                 }
                 Ok(max_user_state) => {
                     // Data found for the given user
@@ -99,8 +101,9 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
                     let fresh_value_to_add = H::hash(&Self::value_to_bytes(&val));
                     update_set.push((stale_label, stale_value_to_add));
                     update_set.push((fresh_label, fresh_value_to_add));
-                    let new_state = UserState::new(val, latest_version, fresh_label, next_epoch);
-                    user_data_update_set.push((uname, new_state));
+                    let new_state =
+                        ValueState::new(uname, val, latest_version, fresh_label, next_epoch);
+                    user_data_update_set.push(new_state);
                 }
             }
         }
@@ -112,10 +115,10 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
             .batch_insert_leaves(&self.storage, insertion_set)
             .await;
         self.storage
-            .store(AzksKey(self.azks_id), &current_azks)
+            .set::<H>(DbRecord::Azks(current_azks.clone()))
             .await?;
         self.storage
-            .append_user_states(user_data_update_set)
+            .append_user_states::<H>(user_data_update_set)
             .await?;
         self.current_epoch = next_epoch;
         output
@@ -123,17 +126,17 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
         // want to change this to call a write operation to post to a blockchain or some such thing
     }
 
-    // Provides proof for correctness of latest version
-    pub async fn lookup(&self, uname: Username) -> Result<LookupProof<H>, SeemlessError> {
+    /// Provides proof for correctness of latest version
+    pub async fn lookup(&self, uname: AkdKey) -> Result<LookupProof<H>, AkdError> {
         match self
             .storage
-            .get_user_state(&uname, UserStateRetrievalFlag::MaxEpoch)
+            .get_user_state::<H>(&uname, ValueStateRetrievalFlag::MaxEpoch)
             .await
         {
             Err(_) => {
                 // Need to throw an error
-                Err(SeemlessError::SeemlessDirectoryErr(
-                    SeemlessDirectoryError::LookedUpNonExistentUser(uname.0, self.current_epoch),
+                Err(AkdError::DirectoryErr(
+                    DirectoryError::LookedUpNonExistentUser(uname.0, self.current_epoch),
                 ))
             }
             Ok(latest_st) => {
@@ -172,9 +175,9 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
     /// this function returns all the values ever associated with it,
     /// and the epoch at which each value was first committed to the server state.
     /// It also returns the proof of the latest version being served at all times.
-    pub async fn key_history(&self, uname: &Username) -> Result<HistoryProof<H>, SeemlessError> {
+    pub async fn key_history(&self, uname: &AkdKey) -> Result<HistoryProof<H>, AkdError> {
         let username = uname.0.to_string();
-        if let Ok(this_user_data) = self.storage.get_user_data(uname).await {
+        if let Ok(this_user_data) = self.storage.get_user_data::<H>(uname).await {
             let mut proofs = Vec::<UpdateProof<H>>::new();
             for user_state in &this_user_data.states {
                 let proof = self.create_single_update_proof(uname, user_state).await?;
@@ -182,47 +185,56 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
             }
             Ok(HistoryProof { proofs })
         } else {
-            Err(SeemlessError::SeemlessDirectoryErr(
-                SeemlessDirectoryError::LookedUpNonExistentUser(username, self.current_epoch),
+            Err(AkdError::DirectoryErr(
+                DirectoryError::LookedUpNonExistentUser(username, self.current_epoch),
             ))
         }
     }
 
-    // Needs error handling in case the epochs are invalid
+    /// Returns an AppendOnlyProof for the leaves inseted into the underlying tree between
+    /// the epochs audit_start_ep and audit_end_ep.
     pub async fn audit(
         &self,
         audit_start_ep: u64,
         audit_end_ep: u64,
-    ) -> Result<AppendOnlyProof<H>, SeemlessError> {
+    ) -> Result<AppendOnlyProof<H>, AkdError> {
         let current_azks = self.retrieve_current_azks().await?;
         current_azks
             .get_append_only_proof(&self.storage, audit_start_ep, audit_end_ep)
             .await
     }
 
-    pub async fn retrieve_current_azks(&self) -> Result<Azks<H, S>, crate::errors::StorageError> {
-        self.storage
-            .retrieve::<Azks<H, S>>(AzksKey(self.azks_id))
-            .await
+    /// Retrieves the current azks
+    pub async fn retrieve_current_azks(&self) -> Result<Azks<H>, crate::errors::StorageError> {
+        Directory::get_azks_from_storage(&self.storage).await
+    }
+
+    async fn get_azks_from_storage(storage: &S) -> Result<Azks<H>, crate::errors::StorageError> {
+        let got = storage
+            .get::<H, Azks<H>>(crate::append_only_zks::DEFAULT_AZKS_KEY)
+            .await?;
+        match got {
+            DbRecord::Azks(azks) => Ok(azks),
+            _ => Err(crate::errors::StorageError::GetError(String::from(
+                "Not found",
+            ))),
+        }
     }
 
     /// HELPERS ///
 
     #[allow(unused)]
-    fn get_azks_id(&self) -> &[u8] {
-        &self.azks_id
-    }
-
-    #[allow(unused)]
-    fn username_to_nodelabel(_uname: &Username) -> NodeLabel {
+    fn username_to_nodelabel(_uname: &AkdKey) -> NodeLabel {
         // this function will need to read the VRF key off some function
         unimplemented!()
     }
 
-    // TODO: we need to make this only work on the server and have another function
+    // FIXME: we need to make this only work on the server, use a VRF and have another function
     // that verifies nodelabel.
-    /// FIXME: Add a comment here for what the stale parameter is used for
-    pub(crate) fn get_nodelabel(uname: &Username, stale: bool, version: u64) -> NodeLabel {
+    /// Returns the tree nodelabel that corresponds to a version of the akdkey argument.
+    /// The stale boolean here is to indicate whether we are getting the nodelabel for a fresh version,
+    /// or a version that we are retiring.
+    pub(crate) fn get_nodelabel(uname: &AkdKey, stale: bool, version: u64) -> NodeLabel {
         // this function will need to read the VRF key using some function
         let name_hash_bytes = H::hash(uname.0.as_bytes());
         let mut stale_bytes = &[1u8];
@@ -239,27 +251,18 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
         NodeLabel::new(u64::from_ne_bytes(hashed_label_bytes), 64u32)
     }
 
+    // FIXME: Make a real commitment here, alongwith a blinding factor.
+    /// Gets the bytes for a value.
     pub fn value_to_bytes(_value: &Values) -> [u8; 64] {
         [0u8; 64]
         // unimplemented!()
     }
 
-    async fn get_azks_from_storage(storage: &S) -> Option<Azks<H, S>> {
-        let result = storage.retrieve_all::<Azks<H, S>>(Some(1)).await;
-        if let Ok(mut v) = result {
-            if !v.is_empty() {
-                let removed = v.remove(0);
-                return Some(removed);
-            }
-        }
-        None
-    }
-
     async fn create_single_update_proof(
         &self,
-        uname: &Username,
-        user_state: &UserState,
-    ) -> Result<UpdateProof<H>, SeemlessError> {
+        uname: &AkdKey,
+        user_state: &ValueState,
+    ) -> Result<UpdateProof<H>, AkdError> {
         let epoch = user_state.epoch;
         let plaintext_value = &user_state.plaintext_val;
         let version = &user_state.version;
@@ -325,20 +328,20 @@ impl<S: Storage + std::marker::Sync + std::marker::Send, H: Hasher + std::marker
         })
     }
 
+    /// Gets the azks root hash at the provided epoch. Note that the root hash should exist at any epoch
+    /// that the azks existed, so as long as epoch >= 0, we should be fine.
     pub async fn get_root_hash_at_epoch(
         &self,
-        current_azks: &Azks<H, S>,
+        current_azks: &Azks<H>,
         epoch: u64,
-    ) -> Result<H::Digest, SeemlessError> {
+    ) -> Result<H::Digest, AkdError> {
         Ok(current_azks
             .get_root_hash_at_epoch(&self.storage, epoch)
             .await?)
     }
 
-    pub async fn get_root_hash(
-        &self,
-        current_azks: &Azks<H, S>,
-    ) -> Result<H::Digest, SeemlessError> {
+    /// Gets the azks root hash at the current epoch.
+    pub async fn get_root_hash(&self, current_azks: &Azks<H>) -> Result<H::Digest, AkdError> {
         self.get_root_hash_at_epoch(current_azks, self.current_epoch)
             .await
     }
@@ -372,20 +375,18 @@ fn get_random_str<R: RngCore + CryptoRng>(rng: &mut R) -> String {
 
 type KeyHistoryHelper<D> = (Vec<D>, Vec<Option<D>>);
 
-pub async fn get_key_history_hashes<
-    S: Storage + std::marker::Sync + std::marker::Send,
-    H: Hasher + std::marker::Send,
->(
-    seemless_dir: &Directory<S, H>,
+/// Gets hashes for key history proofs
+pub async fn get_key_history_hashes<S: V2Storage + Sync + Send, H: Hasher + Sync + Send>(
+    akd_dir: &Directory<S, H>,
     history_proof: &HistoryProof<H>,
-) -> Result<KeyHistoryHelper<H::Digest>, SeemlessError> {
+) -> Result<KeyHistoryHelper<H::Digest>, AkdError> {
     let mut epoch_hash_map: HashMap<u64, H::Digest> = HashMap::new();
 
     let mut root_hashes = Vec::<H::Digest>::new();
     let mut previous_root_hashes = Vec::<Option<H::Digest>>::new();
-    let current_azks = seemless_dir.retrieve_current_azks().await?;
+    let current_azks = akd_dir.retrieve_current_azks().await?;
     for proof in &history_proof.proofs {
-        let hash = seemless_dir
+        let hash = akd_dir
             .get_root_hash_at_epoch(&current_azks, proof.epoch)
             .await?;
         epoch_hash_map.insert(proof.epoch, hash);
@@ -402,7 +403,7 @@ pub async fn get_key_history_hashes<
             previous_root_hashes.push(Some(*hash));
         } else {
             // cache miss, fetch it
-            let hash = seemless_dir
+            let hash = akd_dir
                 .get_root_hash_at_epoch(&current_azks, proof.epoch - 1)
                 .await?;
             previous_root_hashes.push(Some(hash));
@@ -426,132 +427,130 @@ mod tests {
     // FIXME: #[test]
     #[allow(unused)]
     #[actix_rt::test]
-    async fn test_simple_publish() -> Result<(), SeemlessError> {
-        let db = AsyncInMemoryDatabase::new();
-        let mut seemless =
-            Directory::<AsyncInMemoryDatabase, Blake3_256<BaseElement>>::new(&db).await?;
-
-        seemless
-            .publish(vec![(
-                Username("hello".to_string()),
-                Values("world".to_string()),
-            )])
-            .await?;
+    async fn test_simple_publish() -> Result<(), AkdError> {
+        let db = crate::storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut akd = Directory::<
+            crate::storage::V2FromV1StorageWrapper<AsyncInMemoryDatabase>,
+            Blake3_256<BaseElement>,
+        >::new(&db)
+        .await?;
 
+        akd.publish(vec![(
+            AkdKey("hello".to_string()),
+            Values("world".to_string()),
+        )])
+        .await?;
         Ok(())
     }
 
     #[actix_rt::test]
-    async fn test_simiple_lookup() -> Result<(), SeemlessError> {
-        let db = AsyncInMemoryDatabase::new();
-        let mut seemless =
-            Directory::<AsyncInMemoryDatabase, Blake3_256<BaseElement>>::new(&db).await?;
+    async fn test_simiple_lookup() -> Result<(), AkdError> {
+        let db = crate::storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut akd = Directory::<
+            crate::storage::V2FromV1StorageWrapper<AsyncInMemoryDatabase>,
+            Blake3_256<BaseElement>,
+        >::new(&db)
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello".to_string()), Values("world".to_string())),
-                (Username("hello2".to_string()), Values("world2".to_string())),
-            ])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello".to_string()), Values("world".to_string())),
+            (AkdKey("hello2".to_string()), Values("world2".to_string())),
+        ])
+        .await?;
 
-        let lookup_proof = seemless.lookup(Username("hello".to_string())).await?;
-        let current_azks = seemless.retrieve_current_azks().await?;
-        let root_hash = seemless.get_root_hash(&current_azks).await?;
+        let lookup_proof = akd.lookup(AkdKey("hello".to_string())).await?;
+        let current_azks = akd.retrieve_current_azks().await?;
+        let root_hash = akd.get_root_hash(&current_azks).await?;
         lookup_verify::<Blake3_256<BaseElement>>(
             root_hash,
-            Username("hello".to_string()),
+            AkdKey("hello".to_string()),
             lookup_proof,
         )?;
         Ok(())
     }
 
     #[actix_rt::test]
-    async fn test_simple_key_history() -> Result<(), SeemlessError> {
-        let db = AsyncInMemoryDatabase::new();
-        let mut seemless =
-            Directory::<AsyncInMemoryDatabase, Blake3_256<BaseElement>>::new(&db).await?;
+    async fn test_simple_key_history() -> Result<(), AkdError> {
+        let db = crate::storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut akd = Directory::<
+            crate::storage::V2FromV1StorageWrapper<AsyncInMemoryDatabase>,
+            Blake3_256<BaseElement>,
+        >::new(&db)
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello".to_string()), Values("world".to_string())),
-                (Username("hello2".to_string()), Values("world2".to_string())),
-            ])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello".to_string()), Values("world".to_string())),
+            (AkdKey("hello2".to_string()), Values("world2".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello".to_string()), Values("world3".to_string())),
-                (Username("hello2".to_string()), Values("world4".to_string())),
-            ])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello".to_string()), Values("world".to_string())),
+            (AkdKey("hello2".to_string()), Values("world2".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello3".to_string()), Values("world".to_string())),
-                (Username("hello4".to_string()), Values("world2".to_string())),
-            ])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello".to_string()), Values("world3".to_string())),
+            (AkdKey("hello2".to_string()), Values("world4".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![(
-                Username("hello".to_string()),
-                Values("world_updated".to_string()),
-            )])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello3".to_string()), Values("world".to_string())),
+            (AkdKey("hello4".to_string()), Values("world2".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello3".to_string()), Values("world6".to_string())),
-                (
-                    Username("hello4".to_string()),
-                    Values("world12".to_string()),
-                ),
-            ])
-            .await?;
+        akd.publish(vec![(
+            AkdKey("hello".to_string()),
+            Values("world_updated".to_string()),
+        )])
+        .await?;
 
-        let history_proof = seemless.key_history(&Username("hello".to_string())).await?;
+        akd.publish(vec![
+            (AkdKey("hello3".to_string()), Values("world6".to_string())),
+            (AkdKey("hello4".to_string()), Values("world12".to_string())),
+        ])
+        .await?;
+
+        let history_proof = akd.key_history(&AkdKey("hello".to_string())).await?;
         let (root_hashes, previous_root_hashes) =
-            get_key_history_hashes(&seemless, &history_proof).await?;
+            get_key_history_hashes(&akd, &history_proof).await?;
         key_history_verify::<Blake3_256<BaseElement>>(
             root_hashes,
             previous_root_hashes,
-            Username("hello".to_string()),
+            AkdKey("hello".to_string()),
             history_proof,
         )?;
 
-        let history_proof = seemless
-            .key_history(&Username("hello2".to_string()))
-            .await?;
+        let history_proof = akd.key_history(&AkdKey("hello2".to_string())).await?;
         let (root_hashes, previous_root_hashes) =
-            get_key_history_hashes(&seemless, &history_proof).await?;
+            get_key_history_hashes(&akd, &history_proof).await?;
         key_history_verify::<Blake3_256<BaseElement>>(
             root_hashes,
             previous_root_hashes,
-            Username("hello2".to_string()),
+            AkdKey("hello2".to_string()),
             history_proof,
         )?;
 
-        let history_proof = seemless
-            .key_history(&Username("hello3".to_string()))
-            .await?;
+        let history_proof = akd.key_history(&AkdKey("hello3".to_string())).await?;
         let (root_hashes, previous_root_hashes) =
-            get_key_history_hashes(&seemless, &history_proof).await?;
+            get_key_history_hashes(&akd, &history_proof).await?;
         key_history_verify::<Blake3_256<BaseElement>>(
             root_hashes,
             previous_root_hashes,
-            Username("hello3".to_string()),
+            AkdKey("hello3".to_string()),
             history_proof,
         )?;
 
-        let history_proof = seemless
-            .key_history(&Username("hello4".to_string()))
-            .await?;
+        let history_proof = akd.key_history(&AkdKey("hello4".to_string())).await?;
         let (root_hashes, previous_root_hashes) =
-            get_key_history_hashes(&seemless, &history_proof).await?;
+            get_key_history_hashes(&akd, &history_proof).await?;
         key_history_verify::<Blake3_256<BaseElement>>(
             root_hashes,
             previous_root_hashes,
-            Username("hello4".to_string()),
+            AkdKey("hello4".to_string()),
             history_proof,
         )?;
 
@@ -560,95 +559,96 @@ mod tests {
 
     #[allow(unused)]
     #[actix_rt::test]
-    async fn test_simple_audit() -> Result<(), SeemlessError> {
-        let db = AsyncInMemoryDatabase::new();
-        let mut seemless =
-            Directory::<AsyncInMemoryDatabase, Blake3_256<BaseElement>>::new(&db).await?;
+    async fn test_simple_audit() -> Result<(), AkdError> {
+        let db = crate::storage::V2FromV1StorageWrapper::new(AsyncInMemoryDatabase::new());
+        let mut akd = Directory::<
+            crate::storage::V2FromV1StorageWrapper<AsyncInMemoryDatabase>,
+            Blake3_256<BaseElement>,
+        >::new(&db)
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello".to_string()), Values("world".to_string())),
-                (Username("hello2".to_string()), Values("world2".to_string())),
-            ])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello".to_string()), Values("world".to_string())),
+            (AkdKey("hello2".to_string()), Values("world2".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello".to_string()), Values("world3".to_string())),
-                (Username("hello2".to_string()), Values("world4".to_string())),
-            ])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello".to_string()), Values("world".to_string())),
+            (AkdKey("hello2".to_string()), Values("world2".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello3".to_string()), Values("world".to_string())),
-                (Username("hello4".to_string()), Values("world2".to_string())),
-            ])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello".to_string()), Values("world3".to_string())),
+            (AkdKey("hello2".to_string()), Values("world4".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![(
-                Username("hello".to_string()),
-                Values("world_updated".to_string()),
-            )])
-            .await?;
+        akd.publish(vec![
+            (AkdKey("hello3".to_string()), Values("world".to_string())),
+            (AkdKey("hello4".to_string()), Values("world2".to_string())),
+        ])
+        .await?;
 
-        seemless
-            .publish(vec![
-                (Username("hello3".to_string()), Values("world6".to_string())),
-                (
-                    Username("hello4".to_string()),
-                    Values("world12".to_string()),
-                ),
-            ])
-            .await?;
+        akd.publish(vec![(
+            AkdKey("hello".to_string()),
+            Values("world_updated".to_string()),
+        )])
+        .await?;
 
-        let current_azks = seemless.retrieve_current_azks().await?;
+        akd.publish(vec![
+            (AkdKey("hello3".to_string()), Values("world6".to_string())),
+            (AkdKey("hello4".to_string()), Values("world12".to_string())),
+        ])
+        .await?;
 
-        let audit_proof_1 = seemless.audit(1, 2).await?;
+        let current_azks = akd.retrieve_current_azks().await?;
+
+        let audit_proof_1 = akd.audit(1, 2).await?;
         audit_verify::<Blake3_256<BaseElement>>(
-            seemless.get_root_hash_at_epoch(&current_azks, 1).await?,
-            seemless.get_root_hash_at_epoch(&current_azks, 2).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 1).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 2).await?,
             audit_proof_1,
         )
         .await?;
 
-        let audit_proof_2 = seemless.audit(1, 3).await?;
+        let audit_proof_2 = akd.audit(1, 3).await?;
         audit_verify::<Blake3_256<BaseElement>>(
-            seemless.get_root_hash_at_epoch(&current_azks, 1).await?,
-            seemless.get_root_hash_at_epoch(&current_azks, 3).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 1).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 3).await?,
             audit_proof_2,
         )
         .await?;
 
-        let audit_proof_3 = seemless.audit(1, 4).await?;
+        let audit_proof_3 = akd.audit(1, 4).await?;
         audit_verify::<Blake3_256<BaseElement>>(
-            seemless.get_root_hash_at_epoch(&current_azks, 1).await?,
-            seemless.get_root_hash_at_epoch(&current_azks, 4).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 1).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 4).await?,
             audit_proof_3,
         )
         .await?;
 
-        let audit_proof_4 = seemless.audit(1, 5).await?;
+        let audit_proof_4 = akd.audit(1, 5).await?;
         audit_verify::<Blake3_256<BaseElement>>(
-            seemless.get_root_hash_at_epoch(&current_azks, 1).await?,
-            seemless.get_root_hash_at_epoch(&current_azks, 5).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 1).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 5).await?,
             audit_proof_4,
         )
         .await?;
 
-        let audit_proof_5 = seemless.audit(2, 3).await?;
+        let audit_proof_5 = akd.audit(2, 3).await?;
         audit_verify::<Blake3_256<BaseElement>>(
-            seemless.get_root_hash_at_epoch(&current_azks, 2).await?,
-            seemless.get_root_hash_at_epoch(&current_azks, 3).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 2).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 3).await?,
             audit_proof_5,
         )
         .await?;
 
-        let audit_proof_6 = seemless.audit(2, 4).await?;
+        let audit_proof_6 = akd.audit(2, 4).await?;
         audit_verify::<Blake3_256<BaseElement>>(
-            seemless.get_root_hash_at_epoch(&current_azks, 2).await?,
-            seemless.get_root_hash_at_epoch(&current_azks, 4).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 2).await?,
+            akd.get_root_hash_at_epoch(&current_azks, 4).await?,
             audit_proof_6,
         )
         .await?;
