@@ -5,9 +5,11 @@
 // License, Version 2.0 found in the LICENSE-APACHE file in the root directory
 // of this source tree.
 
+//! The representation for the label of a history tree node.
+
 use crate::serialization::from_digest;
 use crate::storage::types::StorageType;
-use crate::storage::{Storable, Storage};
+use crate::storage::Storable;
 use crate::{Direction, ARITY};
 use serde::{Deserialize, Serialize};
 use std::marker::PhantomData;
@@ -19,20 +21,30 @@ use winter_crypto::Hasher;
 
 use rand::{CryptoRng, RngCore};
 
+/// The NodeLabel struct represents the label for a HistoryTreeNode.
+/// Since the label itself may have any number of zeros pre-pended,
+/// just using a native type, unless it is a bit-vector, wouldn't work.
+/// Hence, we need a custom representation.
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub struct NodeLabel {
+    /// val stores a binary string as a u64
     pub val: u64,
+    /// len keeps track of how long the binary string is
     pub len: u32,
 }
 
 impl NodeLabel {
+    /// Creates a new NodeLabel with the given value and len.
     pub fn new(val: u64, len: u32) -> Self {
         NodeLabel { val, len }
     }
 
+    /// Gets the length of a NodeLabel.
     pub fn get_len(&self) -> u32 {
         self.len
     }
+
+    /// Gets the value of a NodeLabel.
     pub fn get_val(&self) -> u64 {
         self.val
     }
@@ -65,6 +77,8 @@ impl NodeLabel {
         Self::new(self.val >> (self.len - len), len)
     }
 
+    /// Takes as input a pointer to the caller and another NodeLabel,
+    /// returns a NodeLabel that is the longest common prefix of the two.
     pub fn get_longest_common_prefix(&self, other: Self) -> Self {
         let shorter_len = if self.get_len() < other.get_len() {
             self.get_len()
@@ -82,6 +96,12 @@ impl NodeLabel {
         self.get_prefix(prefix_len)
     }
 
+    /// Takes as input a pointer to self, another NodeLabel and returns the tuple representing:
+    /// * the longest common prefix,
+    /// * the direction, with respect to the longest common prefix, of other,
+    /// * the direction, with respect to the longest common prefix, of self.
+    /// If either the node itself, or other is the longest common prefix, the
+    /// direction of the longest common prefix node is None.
     pub fn get_longest_common_prefix_and_dirs(&self, other: Self) -> (Self, Direction, Direction) {
         let lcp_label = self.get_longest_common_prefix(other);
         let dir_other = lcp_label.get_dir(other);
@@ -89,6 +109,8 @@ impl NodeLabel {
         (lcp_label, dir_other, dir_self)
     }
 
+    /// Gets the direction of other with respect to self, if self is a prefix of other.
+    /// If self is not a prefix of other, then returns None.
     pub fn get_dir(&self, other: Self) -> Direction {
         if self.get_len() >= other.get_len() {
             return Direction::None;
@@ -100,6 +122,8 @@ impl NodeLabel {
     }
 }
 
+/// Hashes a label of type NodeLabel using the hash function provided by
+/// the generic type H.
 pub fn hash_label<H: Hasher>(label: NodeLabel) -> H::Digest {
     let byte_label_len = H::hash(&label.get_len().to_ne_bytes());
     H::merge_with_int(byte_label_len, label.get_val())
@@ -107,57 +131,71 @@ pub fn hash_label<H: Hasher>(label: NodeLabel) -> H::Digest {
 
 #[derive(Debug, Serialize, Deserialize)]
 #[serde(bound = "")]
-pub struct HistoryNodeState<H, S> {
+/// A HistoryNodeState represents the state of a [crate::history_tree_node::HistoryTreeNode] at a given epoch.
+/// As you may see, when looking at [HistoryChildState], the node needs to include
+/// its hashed value, the hashed values of its children and the labels of its children.
+/// This allows the various algorithms in [crate::history_tree_node::HistoryTreeNode] to build proofs for the tree at
+/// any given epoch, without having to do a traversal of the history tree to find siblings.
+/// The hash value of this node at this state.
+/// To be used in its parent, alongwith the label.
+pub struct HistoryNodeState<H> {
+    /// The hash at this node state
     pub value: Vec<u8>,
-    pub child_states: Vec<HistoryChildState<H, S>>,
+    /// The states of the children at this time
+    pub child_states: Vec<HistoryChildState<H>>,
+    /// A unique key
+    pub key: NodeStateKey,
 }
 
-// parameters are azks_id, node location, and epoch
-#[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
-pub struct NodeStateKey(pub(crate) [u8; 32], pub(crate) NodeLabel, pub(crate) usize);
+/// This struct is just used for storage access purposes.
+/// parameters are azks_id, node location, and epoch
+#[derive(Copy, Clone, Serialize, Deserialize, PartialEq, Eq, Hash, Debug)]
+pub struct NodeStateKey(pub NodeLabel, pub u64);
 
-impl<H: Hasher, S: Storage> Storable for HistoryNodeState<H, S> {
+impl<H: Hasher> Storable for HistoryNodeState<H> {
     type Key = NodeStateKey;
 
     fn data_type() -> StorageType {
         StorageType::HistoryNodeState
     }
+
+    fn get_id(&self) -> NodeStateKey {
+        self.key
+    }
 }
 
-unsafe impl<H: Hasher, S: Storage> Sync for HistoryNodeState<H, S> {}
+unsafe impl<H: Hasher> Sync for HistoryNodeState<H> {}
 
-impl<H: Hasher, S: Storage> HistoryNodeState<H, S> {
-    pub fn new() -> Self {
-        let children = vec![HistoryChildState::<H, S>::new_dummy(); ARITY];
+impl<H: Hasher> HistoryNodeState<H> {
+    /// Creates a new [HistoryNodeState]
+    pub fn new(key: NodeStateKey) -> Self {
+        let children = vec![HistoryChildState::<H>::new_dummy(); ARITY];
         HistoryNodeState {
             value: from_digest::<H>(H::hash(&[0u8])).unwrap(),
             child_states: children,
+            key,
         }
     }
 
-    pub fn get_child_state_in_dir(&self, dir: usize) -> HistoryChildState<H, S> {
+    /// Returns a copy of the child state, in the calling HistoryNodeState in the given direction.
+    pub(crate) fn get_child_state_in_dir(&self, dir: usize) -> HistoryChildState<H> {
         self.child_states[dir].clone()
     }
 }
 
-impl<H: Hasher, S: Storage> Default for HistoryNodeState<H, S> {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
-impl<H: Hasher, S: Storage> Clone for HistoryNodeState<H, S> {
+impl<H: Hasher> Clone for HistoryNodeState<H> {
     fn clone(&self) -> Self {
         Self {
             value: self.value.clone(),
             child_states: self.child_states.clone(),
+            key: self.key,
         }
     }
 }
 
 // To use the `{}` marker, the trait `fmt::Display` must be implemented
 // manually for the type.
-impl<H: Hasher, S: Storage> fmt::Display for HistoryNodeState<H, S> {
+impl<H: Hasher> fmt::Display for HistoryNodeState<H> {
     // This trait requires `fmt` with this exact signature.
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         writeln!(f, "\tvalue = {:?}", self.value).unwrap();
@@ -168,23 +206,36 @@ impl<H: Hasher, S: Storage> fmt::Display for HistoryNodeState<H, S> {
     }
 }
 
+///  Marks whether a child state is real
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
 pub enum DummyChildState {
+    /// If this child is dummy. Usually for children of leaves
     Dummy,
+    /// If this child is real
     Real,
 }
+
+/// This struct represents the state of the child of a node at a given epoch
+/// and contains all the information its parent might need about it in an operation.
+/// The dummy_marker represents whether this child was real or a dummy.
+/// In particular, the children of a leaf node are dummies.
 #[derive(Debug, Serialize, Deserialize)]
-pub struct HistoryChildState<H, S> {
+pub struct HistoryChildState<H> {
+    ///  Tells you whether this child is a dummy
     pub dummy_marker: DummyChildState,
+    /// Says where the child node with this label is located
     pub location: usize,
+    /// Child node's label
     pub label: NodeLabel,
+    /// Child node's hash value
     pub hash_val: Vec<u8>,
+    /// Child node's state this epoch being pointed to here
     pub epoch_version: u64,
-    pub(crate) _h: PhantomData<H>,
-    pub(crate) _s: PhantomData<S>,
+    /// Phantom
+    pub _h: PhantomData<H>,
 }
 
-// parameters are azks_id, node location, epoch, child index
+/// parameters are azks_id, node location, epoch, child index
 #[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub struct ChildStateKey(
     pub(crate) Vec<u8>,
@@ -193,17 +244,10 @@ pub struct ChildStateKey(
     pub(crate) usize,
 );
 
-impl<H: Hasher, S: Storage> Storable for HistoryChildState<H, S> {
-    type Key = ChildStateKey;
+unsafe impl<H: Hasher> Sync for HistoryChildState<H> {}
 
-    fn data_type() -> StorageType {
-        StorageType::HistoryChildState
-    }
-}
-
-unsafe impl<H: Hasher, S: Storage> Sync for HistoryChildState<H, S> {}
-
-impl<H: Hasher, S: Storage> HistoryChildState<H, S> {
+impl<H: Hasher> HistoryChildState<H> {
+    /// Instantiates a new [HistoryChildState] with given label and hash val.
     pub fn new(loc: usize, label: NodeLabel, hash_val: H::Digest, ep: u64) -> Self {
         HistoryChildState {
             dummy_marker: DummyChildState::Real,
@@ -212,10 +256,12 @@ impl<H: Hasher, S: Storage> HistoryChildState<H, S> {
             hash_val: from_digest::<H>(hash_val).unwrap(),
             epoch_version: ep,
             _h: PhantomData,
-            _s: PhantomData,
         }
     }
 
+    /// Creates a dummy [HistoryChildState] to signify a node not having children.
+    /// Used elsewhere to instantiate a leaf node of the
+    /// [crate::history_tree_node::HistoryTreeNode] type.
     pub fn new_dummy() -> Self {
         HistoryChildState {
             dummy_marker: DummyChildState::Dummy,
@@ -224,12 +270,11 @@ impl<H: Hasher, S: Storage> HistoryChildState<H, S> {
             hash_val: from_digest::<H>(H::hash(&[0u8])).unwrap(),
             epoch_version: 0,
             _h: PhantomData,
-            _s: PhantomData,
         }
     }
 }
 
-impl<H: Hasher, S: Storage> Clone for HistoryChildState<H, S> {
+impl<H: Hasher> Clone for HistoryChildState<H> {
     fn clone(&self) -> Self {
         Self {
             dummy_marker: self.dummy_marker,
@@ -238,12 +283,11 @@ impl<H: Hasher, S: Storage> Clone for HistoryChildState<H, S> {
             hash_val: self.hash_val.clone(),
             epoch_version: self.epoch_version,
             _h: PhantomData,
-            _s: PhantomData,
         }
     }
 }
 
-impl<H: Hasher, S: Storage> PartialEq for HistoryChildState<H, S> {
+impl<H: Hasher> PartialEq for HistoryChildState<H> {
     fn eq(&self, other: &Self) -> bool {
         self.dummy_marker == other.dummy_marker
             && self.location == other.location
@@ -253,7 +297,7 @@ impl<H: Hasher, S: Storage> PartialEq for HistoryChildState<H, S> {
     }
 }
 
-impl<H: Hasher, S: Storage> fmt::Display for HistoryChildState<H, S> {
+impl<H: Hasher> fmt::Display for HistoryChildState<H> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
             f,
@@ -266,61 +310,6 @@ impl<H: Hasher, S: Storage> fmt::Display for HistoryChildState<H, S> {
     }
 }
 
-/*
-
-use serde::{ Serializer, Deserializer};
-use winter_utils::{Serializable, Deserializable, SliceReader};
-
-
-fn hash_digest_serialize<S, H>(input: &H::Digest, s: S) -> Result<S::Ok, S::Error>
-where
-    S: Serializer, H: Hasher
-{
-    let mut output = vec![];
-    input.write_into(&mut output);
-    s.serialize_bytes(&output)
-}
-
-pub fn hash_digest_deserialize<'de, D, H>(deserializer: D) -> Result<H::Digest, D::Error>
-where
-    D: Deserializer<'de>, H: Hasher
-{
-    let input: &[u8] = Deserialize::deserialize(deserializer).unwrap();
-    Ok(H::Digest::read_from(&mut SliceReader {
-        source: &input,
-        pos: 0,
-    }).unwrap()) // FIXME
-}
-
-mod hash_digest_serde {
-
-    use serde::{ Serializer, Deserializer, Deserialize};
-    use crypto::{ Hasher };
-    use winter_utils::{Serializable, Deserializable, SliceReader};
-
-    pub fn serialize<S, H>(input: &H::Digest, s: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer, H: Hasher
-    {
-        let mut output = vec![];
-        input.write_into(&mut output);
-        s.serialize_bytes(&output)
-    }
-
-    pub fn deserialize<'de, D, H>(deserializer: D) -> Result<H::Digest, D::Error>
-    where
-        D: Deserializer<'de>, H: Hasher
-    {
-        let input: &[u8] = Deserialize::deserialize(deserializer).unwrap();
-        Ok(H::Digest::read_from(&mut SliceReader {
-            source: &input,
-            pos: 0,
-        }).unwrap()) // FIXME
-    }
-}
-
-*/
-
 #[cfg(test)]
 mod tests {
     use super::*;
